<?php

namespace Tests\Feature;

use Illuminate\Support\Str;
use Laravel\Sanctum\Sanctum;
use Tests\TestCase;
use App\Models\User;
use App\Models\Retailer;
use Hash;

class RetailerUpdateTest extends TestCase
{
    public function test_success_update_retailer_by_admin()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/retailers/'.$retailer->id.'?editing=true&editMode=update', [
            'name' => 'test_name',
        ]);
        $retailer->delete();
        $user->delete();
        $response->assertStatus(200);
    }

    public function test_success_update_retailer_by_admin_without_change_already_exist_name()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/retailers/'.$retailer->id.'?editing=true&editMode=update', [
            'name' => $retailer->name
        ]);
        $retailer->delete();
        $user->delete();
        $response->assertStatus(200);
    }

    public function test_update_retailer_by_admin_without_name_of_category()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/retailers/'.$retailer->id.'?editing=true&editMode=update', [
        ]);
        $retailer->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'name' => 'The Name field is required.'
        ]);
    }

    public function test_update_retailer_by_admin_with_already_exist_name()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $retailer = Retailer::factory()->create();
        $retailerAlreadyExist = Retailer::factory()->create();
        $response = $this->put('nova-api/retailers/'.$retailer->id.'?editing=true&editMode=update', [
            'name' => $retailerAlreadyExist->name
        ]);
        $retailer->delete();
        $retailerAlreadyExist->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'name' => 'The Name has already been taken.'
        ]);
    }

    public function test_update_non_existing_retailer_by_admin()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/retailers/-1?editing=true&editMode=update', [
            'name' => 'test_name'
        ]);
        $user->delete();
        $response->assertStatus(404);
    }

    public function test_success_update_retailer_by_analyst()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/retailers/'.$retailer->id.'?editing=true&editMode=update', [
            'name' => 'test_name',
        ]);
        $retailer->delete();
        $user->delete();
        $response->assertStatus(200);
    }

    public function test_success_update_retailer_by_analyst_without_change_already_exist_name()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/retailers/'.$retailer->id.'?editing=true&editMode=update', [
            'name' => $retailer->name
        ]);
        $retailer->delete();
        $user->delete();
        $response->assertStatus(200);
    }

    public function test_update_retailer_by_analyst_without_name_of_category()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/retailers/'.$retailer->id.'?editing=true&editMode=update', [
        ]);
        $retailer->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'name' => 'The Name field is required.'
        ]);
    }

    public function test_update_retailer_by_analyst_with_already_exist_name()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $retailer = Retailer::factory()->create();
        $retailerAlreadyExist = Retailer::factory()->create();
        $response = $this->put('nova-api/retailers/'.$retailer->id.'?editing=true&editMode=update', [
            'name' => $retailerAlreadyExist->name
        ]);
        $retailer->delete();
        $retailerAlreadyExist->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'name' => 'The Name has already been taken.'
        ]);
    }

    public function test_update_non_existing_retailer_by_analyst()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('nova-api/retailers/-1?editing=true&editMode=update', [
            'name' => 'test_name'
        ]);
        $user->delete();
        $response->assertStatus(404);
    }

    public function test_update_retailer_by_common()
    {
        $user = User::factory()->create([
            'role' => 'common',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $retailer = Retailer::factory()->create();
        $response = $this->put('nova-api/retailers/'.$retailer->id.'?editing=true&editMode=update', [
            'name' => $retailer->name
        ]);
        $retailer->delete();
        $user->delete();
        $response->assertStatus(403);
    }

    public function test_update_retailer_by_unauthenticate_user()
    {
        $retailer = Retailer::factory()->create();
        $response = $this->put('nova-api/retailers/'.$retailer->id.'?editing=true&editMode=update', [
            'name' => $retailer->name
        ]);
        $retailer->delete();
        $response->assertStatus(401);
    }
}

